/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *                                                                             *
 * SCORES                                                                      *
 * Copyright(c) 2009 Mayhem Studio                                             *
 *                                                                             *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *                                                                             *
 *   Package: Mobile Widgets Framework v2                                      *
 *   Support: Opera, JIL                                                       *
 *   Version: 1.0.0                                                            *
 * Developer: Angel                                                            *
 *   Created: 10.Sep.09                                                        *
 *  Modified: 10.Sep.09                                                        *
 *                                                                             *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *                                                                             *
 * Manages the scores storage and retrieval.                                   *
 *                                                                             *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * HISTORY                                                                     *
 *                                                                             *
 * [10.Sep.09] Completed the first working version.                            *
 *                                                                             *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

Scores = { }

Scores.MAX_NUMBER_OF_SCORES = 15

Scores.STR_KEY_SCORES_COUNT = "scores-count"
Scores.STR_KEY_ITEMS_COUNT  = "items-count"
Scores.STR_KEY_SCORE        = "score"
Scores.STR_KEY_ITEM         = "item"

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/*                                Public Methods                               */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

// GET NUMBER OF SCORES
//
Scores.getNumberOfScores = function ()
{
    if (parent.widget)
    {
        var scoresCountStr  = widget.preferenceForKey (Scores.STR_KEY_SCORES_COUNT)

        if (scoresCountStr != "")
        {
            scoresCount = parseInt (scoresCountStr)

            if (!isNaN (scoresCount))
            {
                return (scoresCount)
            }
        }
    }

    return (0)
}

// SAVE SCORE
// scoreData   is expected to be a mixed array of strings and numbers of any length (e.g. ["name", level, score]).
// itemToSort  is optional and is the index of the item of scoreData which will be used to sort the scores.
// reverseSort is optional and is a boolean indicating with true that the array must be sorted from smaller to bigger.
//
Scores.saveScore = function (scoreData, itemToSort, reverseSort)
{
    if (parent.widget)
    {
        // Find the number of scores already saved:

        var scoresCount = Scores.getNumberOfScores ()

        // Load all the scores saved if there's any:

        var scores = scoresCount > 0 ? Scores.getScores () : new Array()

        // Add the current score at the end of the scores list:

        scores[scoresCount++] = scoreData

        // Sort the array of scores if required:

        if (itemToSort && scoresCount > 1)
        {
            scores.sort
            (
                function compare (a, b)
                {
                    var valueA = a[itemToSort]
                    var valueB = b[itemToSort]

                    // Check if both values are integers and treat them as such if it's true:

                    if (!isNaN (parseInt (valueA)) && !isNaN (parseInt (valueB)))
                    {
                        valueA = parseInt (valueA)
                        valueB = parseInt (valueB)
                    }

                    // Comparison valid for integers and strings alike:

                    return (valueA > valueB ? -1 : valueA == valueB ? 0 : +1)
                }
            )

            if (reverseSort)
            {
                scores.reverse ()
            }
        }

        // Save the new scores count:

        widget.setPreferenceForKey ("" + scoresCount, Scores.STR_KEY_SCORES_COUNT)

        // Save the new scores list :

        for (var scoreId = 0; scoreId < scoresCount && scoreId < Scores.MAX_NUMBER_OF_SCORES; scoreId++)
        {
            var itemsCount  = scores[scoreId].length

            widget.setPreferenceForKey ("" + itemsCount, Scores.__getScoreItemsCountKey (scoreId))

            for (var itemId = 0; itemId < itemsCount; itemId++)
            {
                widget.setPreferenceForKey ("" + scores[scoreId][itemId], Scores.__getScoreItemKey (scoreId, itemId))
            }
        }
    }
}

// GET SCORES
//
Scores.getScores = function ()
{
    if (parent.widget)
    {
        var scoresCount = Scores.getNumberOfScores ()

        if (scoresCount > 0)
        {
            var scores  = new Array(scoresCount)

            for (var scoreId = 0; scoreId < scoresCount; scoreId++)
            {
                var itemsCount  = Scores.__getScoreItemsCount (scoreId)
                var scoreItems  = new Array(itemsCount)

                for (var itemId = 0; itemId < itemsCount; itemId++)
                {
                    scoreItems[itemId] = widget.preferenceForKey (Scores.__getScoreItemKey (scoreId, itemId))
                }

                scores[scoreId] = scoreItems
            }

            return (scores)
        }
    }

    return (null)
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/*                               Reserved Methods                              */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

// GET SCORE ITEMS COUNT
//
Scores.__getScoreItemsCount = function (scoreId)
{
    var itemsCountStr  = widget.preferenceForKey (Scores.__getScoreItemsCountKey (scoreId))

    if (itemsCountStr != "")
    {
        itemsCount = parseInt (itemsCountStr)

        if (!isNaN (itemsCount))
        {
            return (itemsCount)
        }
    }

    return (0)
}

// GET SCORE ITEM KEY
// Returns a string used when saving and when loading.
//
Scores.__getScoreItemKey = function (scoreId, itemId)
{
    return (Scores.STR_KEY_SCORE + "-" + scoreId + "." + Scores.STR_KEY_ITEM + "-" + itemId)
}

// GET SCORE ITEMS COUNT KEY
// Returns a string used when saving and when loading.
//
Scores.__getScoreItemsCountKey = function (scoreId, itemId)
{
    return (Scores.STR_KEY_SCORE + "-" + scoreId + "." + Scores.STR_KEY_ITEMS_COUNT)
}
